@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21283 $
- * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Revision: 22553 $
+ * $Author: csoutheren $
+ * $Date: 2009-05-07 08:56:06 +0000 (Thu, 07 May 2009) $
  */
 
 #ifndef OPAL_OPAL_MEDIASTRM_H
@@ -317,7 +317,7 @@ class OpalMediaStream : public PSafeObject
 
     /**Returns true if the media stream is open.
       */
-    bool IsOpen() { return isOpen; }
+    bool IsOpen() const { return isOpen; }
     
     /**Set the patch thread that is using this stream.
       */
@@ -346,7 +346,7 @@ class OpalMediaStream : public PSafeObject
     bool RemoveFilter(const PNotifier & Filter, const OpalMediaFormat & Stage);
 
 #if OPAL_STATISTICS
-    virtual void GetStatistics(OpalMediaStatistics & statistics) const;
+    virtual void GetStatistics(OpalMediaStatistics & statistics, bool fromPatch = false) const;
 #endif
   //@}
 
@@ -528,7 +528,7 @@ class OpalRTPMediaStream : public OpalMediaStream
     { return rtpSession; }
 
 #if OPAL_STATISTICS
-    virtual void GetStatistics(OpalMediaStatistics & statistics) const;
+    virtual void GetStatistics(OpalMediaStatistics & statistics, bool fromPatch = false) const;
 #endif
   //@}
 
@@ -540,7 +540,7 @@ class OpalRTPMediaStream : public OpalMediaStream
 
 
 
-/**This class describes a media stream that transfers data to/from a PChannel.
+/**This class describes a media stream that transfers PCM-16 data to/from a PChannel.
   */
 class OpalRawMediaStream : public OpalMediaStream
 {
@@ -587,8 +587,15 @@ class OpalRawMediaStream : public OpalMediaStream
 
     /**Return the associated PChannel 
      */
-    PChannel * GetChannel() { return channel; }
-    
+    PChannel * GetChannel() { return m_channel; }
+
+    /**Set a new channel for raw PCM stream.
+      */
+    bool SetChannel(
+      PChannel * channel,     ///< New channel
+      bool autoDelete = true  ///< Auto delete channel on exit or replacement
+    );
+
     /**Close the media stream.
 
        Closes the associated PChannel.
@@ -601,12 +608,15 @@ class OpalRawMediaStream : public OpalMediaStream
   //@}
 
   protected:
-    PChannel * channel;
-    bool       autoDelete;
+    PChannel * m_channel;
+    bool       m_autoDelete;
+    PMutex     m_channelMutex;
 
-    PUInt64    averageSignalSum;
-    unsigned   averageSignalSamples;
-    PMutex     averagingMutex;
+    PBYTEArray m_silence;
+
+    PUInt64    m_averageSignalSum;
+    unsigned   m_averageSignalSamples;
+    PMutex     m_averagingMutex;
 
     void CollectAverage(const BYTE * buffer, PINDEX size);
 };
