@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21776 $
+ * $Revision: 22009 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 00:40:05 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2009-02-06 06:03:37 +0000 (Fri, 06 Feb 2009) $
  */
 
 
@@ -323,7 +323,6 @@ class OpalAudioMixer
       protected:
         MixerPCMMap_T channelData;
 
-        DWORD timeStamp;
         PINDEX frameLengthSamples;
         mutable PIntArray mixedData;
         mutable PMutex mutex;
@@ -335,7 +334,6 @@ class OpalAudioMixer
         PBoolean GetStereoFrame(OpalAudioMixerStream::StreamFrame & frame) const;
         PBoolean GetChannelFrame(Key_T key, OpalAudioMixerStream::StreamFrame & frame) const;
         void InsertFrame(Key_T key, OpalAudioMixerStream::StreamFrame & frame);
-        void SetTimestamp(DWORD outputTimestamp) { timeStamp = outputTimestamp; }
 	};
 
   protected:
