@@ -24,13 +24,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
+ * $Revision: 21293 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2008-10-12 23:24:41 +0000 (Sun, 12 Oct 2008) $
  */
 
-#ifndef __OPAL_TRANSCODERS_H
-#define __OPAL_TRANSCODERS_H
+#ifndef OPAL_OPAL_TRANSCODERS_H
+#define OPAL_OPAL_TRANSCODERS_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -545,7 +545,7 @@ class Opal_Empty_##fmt##_Decoder : public OpalEmptyFramedAudioTranscoder \
 OPAL_REGISTER_TRANSCODER(Opal_Empty_##fmt##_Encoder, OpalPCM16, fmt); \
 OPAL_REGISTER_TRANSCODER(Opal_Empty_##fmt##_Decoder, fmt,       OpalPCM16); \
 
-#endif // __OPAL_TRANSCODERS_H
+#endif // OPAL_OPAL_TRANSCODERS_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
