@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 22091 $
- * $Author: csoutheren $
- * $Date: 2009-02-20 05:22:44 +0000 (Fri, 20 Feb 2009) $
+ * $Revision: 22197 $
+ * $Author: rjongbloed $
+ * $Date: 2009-03-12 01:11:14 +0000 (Thu, 12 Mar 2009) $
  */
 
 #ifndef OPAL_OPAL_TRANSPORT_H
@@ -82,9 +82,13 @@ class OpalTransportAddress : public PCaselessString
   /**@name Operations */
   //@{
     /**Determine if the two transport addresses are equivalent.
+       The second parameter indicates if wildcards are allowed. A wildcard
+       IP address is one for which isAny() returns true, a wildcard port
+       is the value 65535.
       */
     PBoolean IsEquivalent(
-      const OpalTransportAddress & address
+      const OpalTransportAddress & address,
+      bool wildcards = false   ///< Allow wildcards
     ) const;
 
     /**Determine if the two transport addresses are compatible.
@@ -322,16 +326,14 @@ class OpalListener : public PObject
     ) const = 0;
 
     /**Get the local transport address on which this listener may be accessed.
+       If remoteAddress is present and is an address that requires NAT for
+       connectivity, then the returned address is adjusted to return the
+       external address and port.
       */
     virtual OpalTransportAddress GetLocalAddress(
-      const OpalTransportAddress & preferredAddress = OpalTransportAddress()
+      const OpalTransportAddress & remoteAddress = OpalTransportAddress()
     ) const = 0;
 
-    /**Get the local transport address on which this listener may be accessed.
-       For backward compatibility with OpenH323, now deprecated.
-      */
-    OpalTransportAddress GetTransportAddress() const { return GetLocalAddress(); }
-
     /**Close channel and wait for associated thread to terminate.
       */
     void CloseWait();
@@ -392,9 +394,12 @@ class OpalListenerIP : public OpalListener
   /**@name Overrides from OpalListener */
   //@{
     /**Get the local transport address on which this listener may be accessed.
+       If remoteAddress is present and is an address that requires NAT for
+       connectivity, then the returned address is adjusted to return the
+       external address and port.
       */
     virtual OpalTransportAddress GetLocalAddress(
-      const OpalTransportAddress & preferredAddress = OpalTransportAddress()
+      const OpalTransportAddress & remoteAddress = OpalTransportAddress()
     ) const;
   //@}
 
@@ -567,6 +572,15 @@ class OpalListenerUDP : public OpalListenerIP
       const OpalTransportAddress & localAddress,
       const OpalTransportAddress & remoteAddress
     ) const;
+
+    /**Get the local transport address on which this listener may be accessed.
+       If remoteAddress is present and is an address that requires NAT for
+       connectivity, then the returned address is adjusted to return the
+       external address and port.
+      */
+    virtual OpalTransportAddress GetLocalAddress(
+      const OpalTransportAddress & remoteAddress = OpalTransportAddress()
+    ) const;
   //@}
 
 
@@ -633,7 +647,9 @@ class OpalTransport : public PIndirectChannel
 
     /**Get the transport dependent name of the local endpoint.
       */
-    virtual OpalTransportAddress GetLocalAddress() const = 0;
+    virtual OpalTransportAddress GetLocalAddress(
+      bool allowNAT = true   ///< Allow translation if remote needs NAT
+    ) const = 0;
 
     /**Set local address to connect from.
        Note that this may not work for all transport types or may work only
@@ -801,7 +817,9 @@ class OpalTransportIP : public OpalTransport
   //@{
     /**Get the transport dependent name of the local endpoint.
       */
-    virtual OpalTransportAddress GetLocalAddress() const;
+    virtual OpalTransportAddress GetLocalAddress(
+      bool allowNAT = true   ///< Allow translation if remote needs NAT
+    ) const;
 
     /**Set local address to connect from.
        Note that this may not work for all transport types or may work only
@@ -994,7 +1012,9 @@ class OpalTransportUDP : public OpalTransportIP
 
     /**Get the transport dependent name of the local endpoint.
       */
-    virtual OpalTransportAddress GetLocalAddress() const;
+    virtual OpalTransportAddress GetLocalAddress(
+      bool allowNAT = true   ///< Allow translation if remote needs NAT
+    ) const;
 
     /**Set local address to connect from.
        Note that this may not work for all transport types or may work only
