@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21860 $
+ * $Revision: 22538 $
  * $Author: rjongbloed $
- * $Date: 2008-12-23 02:24:22 +0000 (Tue, 23 Dec 2008) $
+ * $Date: 2009-05-07 03:43:27 +0000 (Thu, 07 May 2009) $
  */
 
 #ifndef OPAL_SIP_SDP_H
@@ -47,16 +47,32 @@
 
 /////////////////////////////////////////////////////////
 
+class SDPBandwidth : public std::map<PString, unsigned>
+{
+  public:
+    unsigned & operator[](const PString & type);
+    unsigned operator[](const PString & type) const;
+    friend ostream & operator<<(ostream & out, const SDPBandwidth & bw);
+    bool Parse(const PString & param);
+    void SetMin(const PString & type, unsigned value);
+};
+
+/////////////////////////////////////////////////////////
+
+class SDPMediaDescription;
+
 class SDPMediaFormat : public PObject
 {
   PCLASSINFO(SDPMediaFormat, PObject);
   public:
     SDPMediaFormat(
+      SDPMediaDescription & parent,
       RTP_DataFrame::PayloadTypes payloadType,
       const char * name = NULL
     );
 
     SDPMediaFormat(
+      SDPMediaDescription & parent,
       const OpalMediaFormat & mediaFormat
     );
 
@@ -79,14 +95,16 @@ class SDPMediaFormat : public PObject
 
     const OpalMediaFormat & GetMediaFormat() const;
 
-    bool ToNormalisedOptions();
+    bool PreEncode();
+    bool PostDecode(unsigned bandwidth);
 
   protected:
-    virtual void UpdateMediaFormat() const;
+    void InitialiseMediaFormat();
 
-    mutable OpalMediaFormat mediaFormat;
+    OpalMediaFormat mediaFormat;
+
+    SDPMediaDescription & m_parent;
     RTP_DataFrame::PayloadTypes payloadType;
-
     unsigned clockRate;
     PString encodingName;
     PString parameters;
@@ -95,19 +113,6 @@ class SDPMediaFormat : public PObject
 
 PLIST(SDPMediaFormatList, SDPMediaFormat);
 
-
-/////////////////////////////////////////////////////////
-
-class SDPBandwidth : public std::map<PString, unsigned>
-{
-  public:
-    unsigned & operator[](const PString & type);
-    unsigned operator[](const PString & type) const;
-    friend ostream & operator<<(ostream & out, const SDPBandwidth & bw);
-    bool Parse(const PString & param);
-};
-
-
 /////////////////////////////////////////////////////////
 
 class SDPMediaDescription : public PObject
@@ -128,8 +133,9 @@ class SDPMediaDescription : public PObject
       const OpalTransportAddress & address
     );
 
-    virtual void PrintOn(ostream & strm) const;
-    virtual void PrintOn(const OpalTransportAddress & commonAddr, ostream & str) const;
+    virtual bool PreEncode();
+    virtual void Encode(const OpalTransportAddress & commonAddr, ostream & str) const;
+    virtual bool PrintOn(ostream & strm, const PString & str) const;
 
     virtual bool Decode(const PStringArray & tokens);
     virtual bool Decode(char key, const PString & value);
@@ -166,6 +172,8 @@ class SDPMediaDescription : public PObject
     virtual unsigned GetBandwidth(const PString & type) const { return bandwidth[type]; }
     virtual void SetBandwidth(const PString & type, unsigned value) { bandwidth[type] = value; }
 
+    virtual const SDPBandwidth & GetBandwidth() const { return bandwidth; }
+
     virtual void RemoveSDPMediaFormat(const SDPMediaFormat & sdpMediaFormat);
 
     virtual void CreateSDPMediaFormats(const PStringArray & tokens);
@@ -175,8 +183,6 @@ class SDPMediaDescription : public PObject
 
     virtual void ProcessMediaOptions(SDPMediaFormat & sdpFormat, const OpalMediaFormat & mediaFormat);
 
-    virtual bool PrintOn(ostream & strm, const PString & str) const;
-
   protected:
     virtual SDPMediaFormat * FindFormat(PString & str) const;
     virtual void SetPacketTime(const PString & optionName, const PString & value);
@@ -236,6 +242,7 @@ class SDPVideoMediaDescription : public SDPRTPAVPMediaDescription
   public:
     SDPVideoMediaDescription(const OpalTransportAddress & address);
     virtual PString GetSDPMediaType() const;
+    virtual bool PreEncode();
 };
 
 /////////////////////////////////////////////////////////
@@ -303,6 +310,7 @@ class SDPSessionDescription : public PObject
 
     static const PString & ConferenceTotalBandwidthType();
     static const PString & ApplicationSpecificBandwidthType();
+    static const PString & TransportIndependentBandwidthType(); // RFC3890
 
   protected:
     void ParseOwner(const PString & str);
