@@ -36,7 +36,7 @@
 #include <opal.h>
 
 
-#define LOCAL_MEDIA 1
+#define LOCAL_MEDIA 0
 
 
 #if defined(_WIN32)
@@ -61,7 +61,7 @@
 
   #define OPAL_DLL "libopal.so"
 
-  #define OPEN_LIBRARY(name)             dlopen(name, RTLD_NOW)
+  #define OPEN_LIBRARY(name)             dlopen(name, RTLD_NOW|RTLD_GLOBAL)
   #define GET_LIBRARY_FUNCTION(dll, fn)  dlsym(dll, (const char *)(fn));
 
   void * hDLL;
@@ -314,6 +314,10 @@ static void HandleMessages(unsigned timeout)
         }
         break;
 
+      case OpalIndProceeding :
+        puts("Proceeding.\n");
+        break;
+
       case OpalIndAlerting :
         puts("Ringing.\n");
         break;
@@ -494,6 +498,38 @@ int DoSubscribe(const char * package, const char * aor, const char * from)
 }
 
 
+int DoRecord(const char * to, const char * file)
+{
+  // Example cmd line: call 612@ekiga.net
+  OpalMessage command;
+  OpalMessage * response;
+
+
+  printf("Calling %s\n", to);
+
+  memset(&command, 0, sizeof(command));
+  command.m_type = OpalCmdSetUpCall;
+  command.m_param.m_callSetUp.m_partyB = to;
+  if ((response = MySendCommand(&command, "Could not make call")) == NULL)
+    return 0;
+
+  CurrentCallToken = strdup(response->m_param.m_callSetUp.m_callToken);
+  FreeMessageFunction(response);
+
+  printf("Recording %s\n", file);
+
+  memset(&command, 0, sizeof(command));
+  command.m_type = OpalCmdStartRecording;
+  command.m_param.m_recording.m_callToken = CurrentCallToken;
+  command.m_param.m_recording.m_file = file;
+  command.m_param.m_recording.m_channels = 2;
+  if ((response = MySendCommand(&command, "Could not start recording")) == NULL)
+    return 0;
+
+  return 1;
+}
+
+
 typedef enum
 {
   OpListen,
@@ -504,14 +540,15 @@ typedef enum
   OpConsult,
   OpRegister,
   OpSubscribe,
+  OpRecord,
   NumOperations
 } Operations;
 
 static const char * const OperationNames[NumOperations] =
-  { "listen", "call", "mute", "hold", "transfer", "consult", "register", "subscribe" };
+  { "listen", "call", "mute", "hold", "transfer", "consult", "register", "subscribe", "record" };
 
 static int const RequiredArgsForOperation[NumOperations] =
-  { 2, 3, 3, 3, 4, 4, 3, 3 };
+  { 2, 3, 3, 3, 4, 4, 3, 3, 3 };
 
 
 static Operations GetOperation(const char * name)
@@ -540,7 +577,7 @@ int main(int argc, char * argv[])
         fputs(" | ", stderr);
       fputs(OperationNames[op], stderr);
     }
-    fputs(" } [ A-party [ B-party ] ]\n", stderr);
+    fputs(" } [ A-party [ B-party | file ] ]\n", stderr);
     return 1;
   }
 
@@ -623,6 +660,12 @@ int main(int argc, char * argv[])
       HandleMessages(INT_MAX); // More or less forever
       break;
 
+    case OpRecord :
+      if (!DoRecord(argv[2], argv[3]))
+        break;
+      HandleMessages(INT_MAX); // More or less forever
+      break;
+
     default :
       break;
   }
