@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20534 $
+ * $Revision: 22511 $
  * $Author: rjongbloed $
- * $Date: 2008-07-04 03:47:05 +0000 (Fri, 04 Jul 2008) $
+ * $Date: 2009-05-05 05:02:11 +0000 (Tue, 05 May 2009) $
  */
 
 #ifndef _FaxOPAL_MAIN_H
@@ -41,6 +41,28 @@ class MyManager : public OpalManager
 };
 
 
+class MySIPEndPoint : public SIPEndPoint
+{
+    PCLASSINFO(MySIPEndPoint, SIPEndPoint)
+
+  public:
+    MySIPEndPoint(OpalManager & manager) : SIPEndPoint(manager) { }
+    void OnRegistrationStatus(const RegistrationStatus & status);
+
+    PSyncPoint m_completed;
+};
+
+
+class MyFaxEndPoint : public OpalFaxEndPoint
+{
+    PCLASSINFO(MyFaxEndPoint, OpalFaxEndPoint)
+
+  public:
+    MyFaxEndPoint(OpalManager & manager) : OpalFaxEndPoint(manager) { }
+    virtual void OnFaxCompleted(OpalFaxConnection & connection, bool timeout);
+};
+
+
 class FaxOPAL : public PProcess
 {
     PCLASSINFO(FaxOPAL, PProcess)
