@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21752 $
+ * $Revision: 22003 $
  * $Author: rjongbloed $
- * $Date: 2008-12-10 02:48:17 +0000 (Wed, 10 Dec 2008) $
+ * $Date: 2009-02-05 09:32:45 +0000 (Thu, 05 Feb 2009) $
  */
 
 #include "stdafx.h"
@@ -37,6 +37,8 @@
 
 #include <winsock2.h>
 #include <iphlpapi.h>
+#include <pmpolicy.h>
+#include <pm.h>
 
 
 #ifdef _DEBUG
@@ -44,7 +46,7 @@
 #define TRACE_OPTIONS "TraceLevel=4 TraceFile=\\MobileOpalLog.txt"
 //#define TRACE_OPTIONS "TraceLevel=4"
 #else
-#define TRACE_OPTIONS "TraceLevel=4 TraceAppend TraceFile=\\MobileOpalLog.txt"
+#define TRACE_OPTIONS "TraceLevel=3 TraceAppend TraceFile=\\MobileOpalLog.txt"
 #endif
 
 #define SPEAKERMODE_METHOD 3
@@ -301,6 +303,7 @@ CMobileOpalDlg::CMobileOpalDlg(CWnd* pParent /*=NULL*/)
   , m_opal(NULL)
   , m_opalVersion(OPAL_C_API_VERSION)
   , m_speakerphone(false)
+  , m_hPowerRequirement(NULL)
 {
   m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
 }
@@ -587,6 +590,8 @@ void CMobileOpalDlg::InitialiseOPAL()
     }
     OpalFreeMessage(response);
   }
+
+  PowerPolicyNotify(PPN_UNATTENDEDMODE, TRUE);
 }
 
 
@@ -622,7 +627,7 @@ void CMobileOpalDlg::SetStatusText(UINT ids, const char * str)
     }
 
     if (text.IsEmpty())
-    text = str;
+      text = str;
   }
   else {
     if (ids == 0)
@@ -680,6 +685,15 @@ void CMobileOpalDlg::AddRecentCall(const CString & uri)
 }
 
 
+void CMobileOpalDlg::EnableFullPower()
+{
+  m_hPowerRequirement = SetPowerRequirement(_T("WAV1:"), D0, POWER_FORCE|POWER_NAME, NULL, 0);
+  if (m_hPowerRequirement == NULL)
+    MessageBox(_T("Could not set power mode!"));
+  SetSystemPowerState(NULL, POWER_STATE_ON, POWER_FORCE);
+}
+
+
 void CMobileOpalDlg::OnTimer(UINT_PTR nIDEvent)
 {
   if (m_opal == NULL)
@@ -733,11 +747,22 @@ void CMobileOpalDlg::HandleMessage(OpalMessage & message)
     case OpalIndIncomingCall :
       if (m_incomingCallToken.IsEmpty() && m_currentCallToken.IsEmpty()) {
         m_incomingCallToken = message.m_param.m_incomingCall.m_callToken;
-        SetStatusText(IDS_INCOMING_CALL);
+
+        CString text;
+        text.LoadString(IDS_INCOMING_CALL);
+        text += "\r\n";
+        if (*message.m_param.m_incomingCall.m_remoteDisplayName != '\0')
+          text += message.m_param.m_incomingCall.m_remoteDisplayName;
+        else if (*message.m_param.m_incomingCall.m_remoteAddress != '\0')
+          text += message.m_param.m_incomingCall.m_remoteAddress;
+        m_ctrlStatus.SetWindowText(text);
+        m_ctrlStatus.UpdateWindow();
+
         SetCallButton(true, IDS_ANSWER);
         m_ctrlCallAddress.EnableWindow(false);
         ShowWindow(true);
         BringWindowToTop();
+        EnableFullPower();
       }
       else {
         OpalMessage command;
@@ -775,6 +800,12 @@ void CMobileOpalDlg::HandleMessage(OpalMessage & message)
         m_ctrlCallAddress.EnableWindow(true);
 
         SetStatusText(IDS_READY, message.m_param.m_callCleared.m_reason);
+
+        if (m_hPowerRequirement != NULL) {
+          ReleasePowerRequirement(m_hPowerRequirement);
+          m_hPowerRequirement = NULL;
+        }
+        PowerPolicyNotify(PPN_UNATTENDEDMODE, TRUE);
       }
   }
 }
@@ -858,6 +889,7 @@ void CMobileOpalDlg::OnCallAnswer()
     else if (command.m_type == OpalCmdSetUpCall) {
       m_currentCallToken = response->m_param.m_callSetUp.m_callToken;
       AddRecentCall(m_callAddress);
+      EnableFullPower();
     }
     else if (command.m_type == OpalCmdAnswerCall) {
       m_currentCallToken = m_incomingCallToken;
