@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21854 $
- * $Author: csoutheren $
- * $Date: 2008-12-22 02:18:40 +0000 (Mon, 22 Dec 2008) $
+ * $Revision: 22353 $
+ * $Author: rjongbloed $
+ * $Date: 2009-03-31 23:16:15 +0000 (Tue, 31 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -137,26 +137,89 @@ OpalMediaFormatList OpalPCSSEndPoint::GetMediaFormats() const
   return list;
 }
 
-static PBoolean SetDeviceName(const PString & name,
+static bool SetDeviceName(const PString & name,
                           PSoundChannel::Directions dir,
                           PString & result)
 {
-  if (name[0] == '#') {
-    PStringArray devices = PSoundChannel::GetDeviceNames(dir);
-    PINDEX id = name.Mid(1).AsUnsigned();
-    if (id == 0 || id > devices.GetSize())
-      return PFalse;
-    result = devices[id-1];
-  }
-  else {
-    PSoundChannel * pChannel = PSoundChannel::CreateChannelByName(name, dir);
-    if (pChannel == NULL)
-      return PFalse;
-    delete pChannel;
+  // First see if the channel can be created from the name alone, this
+  // picks up *.wav style names.
+  PSoundChannel * channel = PSoundChannel::CreateChannelByName(name, dir);
+  if (channel != NULL) {
+    delete channel;
     result = name;
+    return true;
   }
 
-  return PTrue;
+  // Otherwise, look for a partial match.
+
+  // Need to create unique names sans driver
+  PStringList uniqueNames;
+  PStringArray devices = PSoundChannel::GetDeviceNames(dir);
+  for (PINDEX i = 0; i < devices.GetSize(); ++i) {
+    PCaselessString deviceName = devices[i];
+    PINDEX tab = deviceName.Find('\t');
+    if (tab != P_MAX_INDEX)
+      deviceName.Delete(0, tab+1);
+    if (uniqueNames.GetValuesIndex(deviceName) == P_MAX_INDEX)
+      uniqueNames.AppendString(deviceName);
+  }
+
+  int partialMatch = -1;
+  for (PINDEX index = 0; index < uniqueNames.GetSize(); index++) {
+    PCaselessString deviceName = uniqueNames[index];
+
+    // Perfect match?
+    if (deviceName == name) {
+      result = deviceName;
+      return true;
+    }
+
+    // Partial match?
+    if (deviceName.NumCompare(name) == PObject::EqualTo) {
+      if (partialMatch == -1)
+        partialMatch = index;
+      else
+        partialMatch = -2;
+    }
+  }
+
+  // Not a partial match or ambiguous partial
+  if (partialMatch < 0)
+    return false;
+
+  result = uniqueNames[partialMatch];
+  return true;
+}
+
+
+static bool SetDeviceNames(const PString & remoteParty, PString & playResult, PString & recordResult, const char * PTRACE_PARAM(operation))
+{
+  PINDEX prefixLength = remoteParty.Find(':')+1;
+
+  PString playDevice, recordDevice;
+  PINDEX separator = remoteParty.FindOneOf("\n\\", prefixLength);
+  if (separator == P_MAX_INDEX)
+    playDevice = recordDevice = remoteParty.Mid(prefixLength);
+  else {
+    playDevice = remoteParty(prefixLength, separator-1);
+    recordDevice = remoteParty.Mid(separator+1);
+  }
+
+  if (playDevice.IsEmpty() || playDevice == "*")
+    playDevice = playResult;
+  if (!SetDeviceName(playDevice, PSoundChannel::Player, playResult)) {
+    PTRACE(2, "PCSS\tSound player device \"" << playDevice << "\" does not exist, " << operation << " aborted.");
+    return false;
+  }
+
+  if (recordDevice.IsEmpty() || recordDevice == "*")
+    recordDevice = recordResult;
+  if (!SetDeviceName(recordDevice, PSoundChannel::Recorder, recordResult)) {
+    PTRACE(2, "PCSS\tSound recording device \"" << recordDevice << "\" does not exist, " << operation << " aborted.");
+    return false;
+  }
+
+  return true;
 }
 
 
@@ -166,42 +229,12 @@ PBoolean OpalPCSSEndPoint::MakeConnection(OpalCall & call,
                                unsigned int options,
                                OpalConnection::StringOptions * stringOptions)
 {
-  // First strip of the prefix if present
-  PINDEX prefixLength = 0;
-  if (remoteParty.Find(GetPrefixName()+":") == 0)
-    prefixLength = GetPrefixName().GetLength()+1;
-
-  PString playDevice;
-  PString recordDevice;
-  PINDEX separator = remoteParty.FindOneOf("\n\t\\", prefixLength);
-  if (separator == P_MAX_INDEX)
-    playDevice = recordDevice = remoteParty.Mid(prefixLength);
-  else {
-    playDevice = remoteParty(prefixLength+1, separator-1);
-    recordDevice = remoteParty.Mid(separator+1);
-  }
-
-  if (!SetDeviceName(playDevice, PSoundChannel::Player, playDevice))
-    playDevice = soundChannelPlayDevice;
-  if (!SetDeviceName(recordDevice, PSoundChannel::Recorder, recordDevice))
-    recordDevice = soundChannelRecordDevice;
-
-  // Make sure sound devices are available.
-  PSoundChannel * soundChannel = PSoundChannel::CreateChannelByName(playDevice, PSoundChannel::Player);
-  if (soundChannel == NULL) {
-    PTRACE(2, "PCSS\tSound player device \"" << playDevice << "\" in use, call " << call << " aborted.");
+  PString playDevice = soundChannelPlayDevice;
+  PString recordDevice = soundChannelRecordDevice;
+  if (!SetDeviceNames(remoteParty, playDevice, recordDevice, "call")) {
     call.Clear(OpalConnection::EndedByLocalBusy);
     return false;
   }
-  delete soundChannel;
-
-  soundChannel = PSoundChannel::CreateChannelByName(recordDevice, PSoundChannel::Recorder);
-  if (soundChannel == NULL) {
-    PTRACE(2, "PCSS\tSound recording device \"" << recordDevice << "\" in use, call " << call << " aborted.");
-    call.Clear(OpalConnection::EndedByLocalBusy);
-    return false;
-  }
-  delete soundChannel;
 
   return AddConnection(CreateConnection(call, playDevice, recordDevice, userData, options, stringOptions));
 }
@@ -249,12 +282,14 @@ PSoundChannel * OpalPCSSEndPoint::CreateSoundChannel(const OpalPCSSConnection &
   }
 
   unsigned channels = mediaFormat.GetOptionInteger(OpalAudioFormat::ChannelsOption());
+  unsigned clockRate = mediaFormat.GetClockRate();
 
-  if (soundChannel->Open(deviceName, dir, channels, mediaFormat.GetClockRate(), 16)) {
+  if (soundChannel->Open(deviceName, dir, channels, clockRate, 16)) {
     PTRACE(3, "PCSS\tOpened " 
                << ((channels == 1) ? "mono" : ((channels == 2) ? "stereo" : "multi-channel")) 
                << " sound channel \"" << deviceName
-               << "\" for " << (isSource ? "record" : "play") << "ing.");
+               << "\" for " << (isSource ? "record" : "play") << "ing at "
+               << clockRate/1000 << '.' << (clockRate%1000)/100 << " kHz.");
     return soundChannel;
   }
 
@@ -323,15 +358,13 @@ void OpalPCSSEndPoint::SetSoundChannelBufferDepth(unsigned depth)
 
 /////////////////////////////////////////////////////////////////////////////
 
-static unsigned LastConnectionTokenID;
-
 OpalPCSSConnection::OpalPCSSConnection(OpalCall & call,
                                        OpalPCSSEndPoint & ep,
                                        const PString & playDevice,
                                        const PString & recordDevice,
                                        unsigned options,
                           OpalConnection::StringOptions * stringOptions)
-  : OpalConnection(call, ep, psprintf("%u", ++LastConnectionTokenID), options, stringOptions),
+  : OpalConnection(call, ep, ep.GetManager().GetNextCallToken(), options, stringOptions),
     endpoint(ep),
     soundChannelPlayDevice(playDevice),
     soundChannelRecordDevice(recordDevice),
@@ -340,7 +373,8 @@ OpalPCSSConnection::OpalPCSSConnection(OpalCall & call,
   silenceDetector = new OpalPCM16SilenceDetector(endpoint.GetManager().GetSilenceDetectParams());
   echoCanceler = new OpalEchoCanceler;
 
-  PTRACE(4, "PCSS\tCreated PC sound system connection with token '" << callToken << "'");
+  PTRACE(4, "PCSS\tCreated PC sound system connection: token=\"" << callToken << "\" "
+            "player=\"" << playDevice << "\" recorder=\"" << recordDevice << '"');
 }
 
 
@@ -388,6 +422,34 @@ PBoolean OpalPCSSConnection::SetAlerting(const PString & calleeName, PBoolean)
 }
 
 
+bool OpalPCSSConnection::TransferConnection(const PString & remoteParty)
+{
+  PString playDevice = soundChannelPlayDevice;
+  PString recordDevice = soundChannelRecordDevice;
+  if (!SetDeviceNames(remoteParty, playDevice, recordDevice, "transfer"))
+    return false;
+
+  if ((playDevice *= soundChannelPlayDevice) &&
+      (recordDevice *= soundChannelRecordDevice)) {
+    PTRACE(2, "PCSS\tTransfer to same sound devices, ignoring.");
+    return true;
+  }
+
+  soundChannelPlayDevice = playDevice;
+  soundChannelRecordDevice = recordDevice;
+
+  PTRACE(3, "PCSS\tTransfer to sound devices: play=\"" << playDevice << "\", record=\"" << recordDevice << '"');
+
+  // Now we be really sneaky and just change the sound devices in the OpalAudioMediaStream
+  for (OpalMediaStreamPtr mediaStream = mediaStreams; mediaStream != NULL; ++mediaStream) {
+    OpalRawMediaStream * rawStream = dynamic_cast<OpalRawMediaStream *>(&*mediaStream);
+    if (rawStream != NULL)
+      rawStream->SetChannel(CreateSoundChannel(rawStream->GetMediaFormat(), rawStream->IsSource()));
+  }
+  return true;
+}
+
+
 OpalMediaStream * OpalPCSSConnection::CreateMediaStream(const OpalMediaFormat & mediaFormat,
                                                         unsigned sessionID,
                                                         PBoolean isSource)
