@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 22091 $
- * $Author: csoutheren $
- * $Date: 2009-02-20 05:22:44 +0000 (Fri, 20 Feb 2009) $
+ * $Revision: 22426 $
+ * $Author: rjongbloed $
+ * $Date: 2009-04-14 09:24:21 +0000 (Tue, 14 Apr 2009) $
  */
 
 #include <ptlib.h>
@@ -95,7 +95,7 @@ OpalTransportAddress::OpalTransportAddress(const PString & str,
 
 
 OpalTransportAddress::OpalTransportAddress(const PIPSocket::Address & addr, WORD port, const char * proto)
-  : PCaselessString(addr.AsString(true))
+  : PCaselessString(addr.IsAny() ? PString('*') : addr.AsString(true))
 {
   SetInternalTransport(port, proto);
 }
@@ -108,9 +108,9 @@ PString OpalTransportAddress::GetHostName() const
 
   return transport->GetHostName(*this);
 }
-  
 
-PBoolean OpalTransportAddress::IsEquivalent(const OpalTransportAddress & address) const
+
+PBoolean OpalTransportAddress::IsEquivalent(const OpalTransportAddress & address, bool wildcard) const
 {
   if (*this == address)
     return PTrue;
@@ -122,8 +122,8 @@ PBoolean OpalTransportAddress::IsEquivalent(const OpalTransportAddress & address
   WORD port1 = 65535, port2 = 65535;
   return GetIpAndPort(ip1, port1) &&
          address.GetIpAndPort(ip2, port2) &&
-         (ip1.IsAny() || ip2.IsAny() || (ip1 *= ip2)) &&
-         (port1 == 65535 || port2 == 65535 || port1 == port2);
+         ((ip1 *= ip2) || (wildcard && (ip1.IsAny() || ip2.IsAny()))) &&
+         (port1 == port2 || (wildcard && (port1 == 65535 || port2 == 65535)));
 }
 
 
@@ -220,7 +220,7 @@ void OpalTransportAddress::SetInternalTransport(WORD port, const char * proto)
   }
 
   // use factory to create transport types
-  transport = PFactory<OpalInternalTransport>::CreateInstance(Left(dollar+1));
+  transport = PFactory<OpalInternalTransport>::CreateInstance(Left(dollar+1).ToLower());
   if (transport == NULL)
     return;
 
@@ -530,35 +530,22 @@ OpalListenerIP::OpalListenerIP(OpalEndPoint & endpoint,
 }
 
 
-OpalTransportAddress OpalListenerIP::GetLocalAddress(const OpalTransportAddress & preferredAddress) const
+OpalTransportAddress OpalListenerIP::GetLocalAddress(const OpalTransportAddress & remoteAddress) const
 {
-  PString addr;
+  PIPSocket::Address localIP = localAddress;
 
-  // If specifically bound to interface use that
-  if (!localAddress.IsAny())
-    addr = localAddress.AsString(true);
-  else {
-    // If bound to all, then use '*' unless a preferred address is specified
-    addr = "*";
-
-    PIPSocket::Address ip;
-    if (preferredAddress.GetIpAddress(ip)) {
-      // Verify preferred address is actually an interface in this machine!
-      PIPSocket::InterfaceTable interfaces;
-      if (PIPSocket::GetInterfaceTable(interfaces)) {
-        for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
-          if (interfaces[i].GetAddress() == ip) {
-            addr = ip.AsString(true);
-            break;
-          }
-        }
-      }
+  PIPSocket::Address remoteIP;
+  if (remoteAddress.GetIpAddress(remoteIP)) {
+    OpalManager & manager = endpoint.GetManager();
+    PNatMethod * natMethod = manager.GetNatMethod(remoteIP);
+    if (natMethod != NULL) {
+      if (localIP.IsAny())
+        natMethod->GetInterfaceAddress(localIP);
+      manager.TranslateIPAddress(localIP, remoteIP);
     }
   }
 
-  addr.sprintf(":%u", listenerPort);
-
-  return GetProtoPrefix() + addr;
+  return OpalTransportAddress(localIP, listenerPort, GetProtoPrefix());
 }
 
 
@@ -785,6 +772,27 @@ const char * OpalListenerUDP::GetProtoPrefix() const
 }
 
 
+OpalTransportAddress OpalListenerUDP::GetLocalAddress(const OpalTransportAddress & remoteAddress) const
+{
+  PIPSocket::Address localIP = PIPSocket::GetDefaultIpAny();
+  WORD port = listenerPort;
+
+  PIPSocket::Address remoteIP;
+  if (remoteAddress.GetIpAddress(remoteIP)) {
+    PNatMethod * natMethod = endpoint.GetManager().GetNatMethod(remoteIP);
+    if (natMethod != NULL) {
+      natMethod->GetInterfaceAddress(localIP);
+      listenerBundle->GetAddress(localIP.AsString(), localIP, port, true);
+    }
+  }
+
+  if (localIP.IsAny())
+    listenerBundle->GetAddress(PString::Empty(), localIP, port, false);
+
+  return OpalTransportAddress(localIP, port, GetProtoPrefix());
+}
+
+
 //////////////////////////////////////////////////////////////////////////
 
 OpalTransport::OpalTransport(OpalEndPoint & end)
@@ -919,7 +927,7 @@ OpalTransportIP::OpalTransportIP(OpalEndPoint & end,
 }
 
 
-OpalTransportAddress OpalTransportIP::GetLocalAddress() const
+OpalTransportAddress OpalTransportIP::GetLocalAddress(bool /*allowNAT*/) const
 {
   return OpalTransportAddress(localAddress, localPort, GetProtoPrefix());
 }
@@ -953,6 +961,7 @@ PBoolean OpalTransportIP::SetRemoteAddress(const OpalTransportAddress & address)
   if (IsCompatibleTransport(address))
     return address.GetIpAndPort(remoteAddress, remotePort);
 
+  PTRACE(2, "OpalIP\tAttempt to set incompatible transport " << address);
   return PFalse;
 }
 
@@ -1266,15 +1275,15 @@ bool OpalTransportUDP::SetInterface(const PString & iface)
 }
 
 
-OpalTransportAddress OpalTransportUDP::GetLocalAddress() const
+OpalTransportAddress OpalTransportUDP::GetLocalAddress(bool allowNAT) const
 {
   PMonitoredSocketChannel * socket = (PMonitoredSocketChannel *)readChannel;
   if (socket != NULL) {
     OpalTransportUDP * thisWritable = const_cast<OpalTransportUDP *>(this);
-    socket->GetLocal(thisWritable->localAddress, thisWritable->localPort, !manager.IsLocalAddress(remoteAddress));
+    socket->GetLocal(thisWritable->localAddress, thisWritable->localPort, allowNAT && !manager.IsLocalAddress(remoteAddress));
   }
 
-  return OpalTransportIP::GetLocalAddress();
+  return OpalTransportIP::GetLocalAddress(allowNAT);
 }
 
 
