@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22147 $
+ * $Revision: 22538 $
  * $Author: rjongbloed $
- * $Date: 2009-03-04 01:56:47 +0000 (Wed, 04 Mar 2009) $
+ * $Date: 2009-05-07 03:43:27 +0000 (Thu, 07 May 2009) $
  */
 
 #include <ptlib.h>
@@ -46,6 +46,8 @@
 #define SIP_DEFAULT_SESSION_NAME    "Opal SIP Session"
 #define SDP_MEDIA_TRANSPORT_RTPAVP  "RTP/AVP"
 
+static const char SDPBandwidthPrefix[] = "SDP-Bandwidth-";
+
 #define new PNEW
 
 //
@@ -163,8 +165,8 @@ SDPMediaFormat::SDPMediaFormat(SDPMediaDescription & parent, RTP_DataFrame::Payl
 
 
 SDPMediaFormat::SDPMediaFormat(SDPMediaDescription & parent, const OpalMediaFormat & fmt)
-  : m_parent(parent) 
-  , mediaFormat(fmt)
+  : mediaFormat(fmt)
+  , m_parent(parent) 
   , payloadType(fmt.GetPayloadType())
   , clockRate(fmt.GetClockRate())
   , encodingName(fmt.GetEncodingName())
@@ -279,7 +281,6 @@ PString SDPMediaFormat::GetFMTP() const
 void SDPMediaFormat::PrintOn(ostream & strm) const
 {
   PAssert(!encodingName.IsEmpty(), "SDPMediaFormat encoding name is empty");
-  mediaFormat.ToCustomisedOptions();
 
   PINDEX i;
   for (i = 0; i < 2; ++i) {
@@ -308,46 +309,56 @@ void SDPMediaFormat::PrintOn(ostream & strm) const
   }
 }
 
-void SDPMediaFormat::UpdateMediaFormat() const
-{
-  mediaFormat = OpalMediaFormat(payloadType, clockRate, encodingName, "sip");
-  if (mediaFormat.IsEmpty())
-    mediaFormat = OpalMediaFormat(encodingName);
-}
 
 const OpalMediaFormat & SDPMediaFormat::GetMediaFormat() const
 {
-  if (mediaFormat.IsEmpty()) {
-
-    UpdateMediaFormat();
-
-    PTRACE_IF(2, mediaFormat.IsEmpty(), "SDP\tCould not find media format for \""
-              << encodingName << "\", pt=" << payloadType << ", clock=" << clockRate);
-
-    mediaFormat.MakeUnique();
-    mediaFormat.SetPayloadType(payloadType);
-
-    if (!parameters.IsEmpty() && (mediaFormat.GetMediaType() == OpalMediaType::Audio())) 
-      mediaFormat.SetOptionInteger(OpalAudioFormat::ChannelsOption(), parameters.AsUnsigned());
-    else
-      mediaFormat.SetOptionInteger(OpalAudioFormat::ChannelsOption(), 1);
-
-    // Fill in the default values for (possibly) missing FMTP options
-    for (PINDEX i = 0; i < mediaFormat.GetOptionCount(); i++) {
-      OpalMediaOption & option = const_cast<OpalMediaOption &>(mediaFormat.GetOption(i));
-      if (!option.GetFMTPName().IsEmpty() && !option.GetFMTPDefault().IsEmpty())
-        option.FromString(option.GetFMTPDefault());
-    }
-
-    for (SDPBandwidth::const_iterator r = m_parent.GetBandwidth().begin(); r != m_parent.GetBandwidth().end(); ++r) 
-      mediaFormat.AddOption(new OpalMediaOptionString("Bandwidth-" & r->first, false, r->second), PTrue);
-  }
-
+  if (mediaFormat.IsEmpty())
+    const_cast<SDPMediaFormat *>(this)->InitialiseMediaFormat();
   return mediaFormat;
 }
 
 
-bool SDPMediaFormat::ToNormalisedOptions()
+void SDPMediaFormat::InitialiseMediaFormat()
+{
+  mediaFormat = OpalMediaFormat(payloadType, clockRate, encodingName, "sip");
+  if (mediaFormat.IsEmpty())
+    mediaFormat = OpalMediaFormat(encodingName);
+  if (mediaFormat.IsEmpty()) {
+    PTRACE(2, "SDP\tCould not find media format for \""
+           << encodingName << "\", pt=" << payloadType << ", clock=" << clockRate);
+    return;
+  }
+
+  mediaFormat.MakeUnique();
+  mediaFormat.SetPayloadType(payloadType);
+
+  if (!parameters.IsEmpty() && (mediaFormat.GetMediaType() == OpalMediaType::Audio())) 
+    mediaFormat.SetOptionInteger(OpalAudioFormat::ChannelsOption(), parameters.AsUnsigned());
+  else
+    mediaFormat.SetOptionInteger(OpalAudioFormat::ChannelsOption(), 1);
+
+  // Fill in the default values for (possibly) missing FMTP options
+  for (PINDEX i = 0; i < mediaFormat.GetOptionCount(); i++) {
+    OpalMediaOption & option = const_cast<OpalMediaOption &>(mediaFormat.GetOption(i));
+    if (!option.GetFMTPName().IsEmpty() && !option.GetFMTPDefault().IsEmpty())
+      option.FromString(option.GetFMTPDefault());
+  }
+
+  for (SDPBandwidth::const_iterator r = m_parent.GetBandwidth().begin(); r != m_parent.GetBandwidth().end(); ++r) {
+    if (r->second > 0)
+      mediaFormat.AddOption(new OpalMediaOptionString(SDPBandwidthPrefix + r->first, false, r->second), true);
+  }
+}
+
+
+bool SDPMediaFormat::PreEncode()
+{
+  mediaFormat.SetOptionString(OpalMediaFormat::ProtocolOption(), "SIP");
+  return mediaFormat.ToCustomisedOptions();
+}
+
+
+bool SDPMediaFormat::PostDecode(unsigned bandwidth)
 {
   if (GetMediaFormat().IsEmpty()) // Use GetMediaFormat() to force creation of member
     return false;
@@ -356,6 +367,12 @@ bool SDPMediaFormat::ToNormalisedOptions()
   if (encodingName.IsEmpty())
     encodingName = mediaFormat.GetEncodingName();
 
+  if (bandwidth > 0) {
+    PTRACE(4, "SDP\tAdjusting format \"" << mediaFormat << "\" bandwidth to " << bandwidth);
+    mediaFormat.SetOptionInteger(OpalMediaFormat::MaxBitRateOption(), bandwidth);
+  }
+
+  mediaFormat.SetOptionString(OpalMediaFormat::ProtocolOption(), "SIP");
   if (mediaFormat.ToNormalisedOptions())
     return true;
 
@@ -412,6 +429,16 @@ bool SDPBandwidth::Parse(const PString & param)
 }
 
 
+void SDPBandwidth::SetMin(const PString & type, unsigned value)
+{
+  iterator it = find(type);
+  if (it == end())
+    (*this)[type] = value;
+  else if (it->second > value)
+    it->second = value;
+}
+
+
 //////////////////////////////////////////////////////////////////////////////
 
 SDPMediaDescription::SDPMediaDescription(const OpalTransportAddress & address)
@@ -548,9 +575,13 @@ bool SDPMediaDescription::Decode(char key, const PString & value)
 
 bool SDPMediaDescription::PostDecode()
 {
+  unsigned bw = bandwidth[SDPSessionDescription::TransportIndependentBandwidthType()];
+  if (bw == 0)
+    bw = bandwidth[SDPSessionDescription::ApplicationSpecificBandwidthType()]*1000;
+
   SDPMediaFormatList::iterator format = formats.begin();
   while (format != formats.end()) {
-    if (format->ToNormalisedOptions())
+    if (format->PostDecode(bw))
       ++format;
     else
       formats.erase(format++);
@@ -652,20 +683,27 @@ void SDPMediaDescription::SetPacketTime(const PString & optionName, const PStrin
 }
 
 
-void SDPMediaDescription::PrintOn(const OpalTransportAddress & commonAddr, ostream & str) const
+bool SDPMediaDescription::PreEncode()
+{
+  for (SDPMediaFormatList::iterator format = formats.begin(); format != formats.end(); ++format) {
+    if (!format->PreEncode())
+      return false;
+  }
+  return true;
+}
+
+
+void SDPMediaDescription::Encode(const OpalTransportAddress & commonAddr, ostream & strm) const
 {
   PString connectString;
   PIPSocket::Address commonIP, transportIP;
   if (transportAddress.GetIpAddress(transportIP) && commonAddr.GetIpAddress(commonIP) && commonIP != transportIP)
     connectString = GetConnectAddressString(transportAddress);
-
-  PrintOn(str, connectString);
+  if (connectString.IsEmpty())
+    connectString = GetConnectAddressString(transportAddress);
+  PrintOn(strm, connectString);
 }
 
-void SDPMediaDescription::PrintOn(ostream & str) const
-{
-  PrintOn(str, GetConnectAddressString(transportAddress));
-}
 
 bool SDPMediaDescription::PrintOn(ostream & str, const PString & connectString) const
 {
@@ -681,14 +719,13 @@ bool SDPMediaDescription::PrintOn(ostream & str, const PString & connectString)
   WORD port = 0;
   transportAddress.GetIpAndPort(ip, port);
 
-  // output media header
+  /* output media header, note the order is important according to RFC!
+     Must be icbka */
   str << "m=" 
-      << GetSDPMediaType() << " "
-      << port << " "
+      << GetSDPMediaType() << ' '
+      << port << ' '
       << GetSDPTransportType()
-      << bandwidth;
-
-  str << GetSDPPortList() << "\r\n";
+      << GetSDPPortList() << "\r\n";
 
   if (!connectString.IsEmpty())
     str << "c=" << connectString << "\r\n";
@@ -697,6 +734,8 @@ bool SDPMediaDescription::PrintOn(ostream & str, const PString & connectString)
   if (port == 0)
     return false;
 
+  str << bandwidth;
+
   // media format direction
   switch (direction) {
     case SDPMediaDescription::RecvOnly:
@@ -960,6 +999,39 @@ PString SDPVideoMediaDescription::GetSDPMediaType() const
   return "video"; 
 }
 
+
+bool SDPVideoMediaDescription::PreEncode()
+{
+  if (!SDPRTPAVPMediaDescription::PreEncode())
+    return false;
+
+  /* Even though the bandwidth parameter COULD be used for audio, we don't
+     do it as it has very limited use. Variable bit rate audio codecs are
+     not common, and usually there is an fmtp value to select the different
+     bit rates. So, as it might cause interoperabity difficulties with the
+     dumber implementations out there we just don't.
+
+     As per RFC3890 we set both AS and TIAS parameters.
+  */
+  for (SDPMediaFormatList::iterator format = formats.begin(); format != formats.end(); ++format) {
+    const OpalMediaFormat & mediaFormat = format->GetMediaFormat();
+
+    for (PINDEX i = 0; i < mediaFormat.GetOptionCount(); ++i) {
+      const OpalMediaOption & option = mediaFormat.GetOption(i);
+      PCaselessString name = option.GetName();
+      if (name.NumCompare(SDPBandwidthPrefix, sizeof(SDPBandwidthPrefix)-1) == EqualTo)
+        bandwidth.SetMin(name.Mid(sizeof(SDPBandwidthPrefix)-1), option.AsString().AsUnsigned());
+    }
+
+    unsigned bw = mediaFormat.GetBandwidth();
+    bandwidth.SetMin(SDPSessionDescription::TransportIndependentBandwidthType(), bw);
+    bandwidth.SetMin(SDPSessionDescription::ApplicationSpecificBandwidthType(), (bw+999)/1000);
+  }
+
+  return true;
+}
+
+
 //////////////////////////////////////////////////////////////////////////////
 
 SDPApplicationMediaDescription::SDPApplicationMediaDescription(const OpalTransportAddress & address)
@@ -997,8 +1069,9 @@ PString SDPApplicationMediaDescription::GetSDPPortList() const
 
 //////////////////////////////////////////////////////////////////////////////
 
-const PString & SDPSessionDescription::ConferenceTotalBandwidthType()     { static PString s = "CT"; return s; }
-const PString & SDPSessionDescription::ApplicationSpecificBandwidthType() { static PString s = "AS"; return s; }
+const PString & SDPSessionDescription::ConferenceTotalBandwidthType()      { static PString s = "CT";   return s; }
+const PString & SDPSessionDescription::ApplicationSpecificBandwidthType()  { static PString s = "AS";   return s; }
+const PString & SDPSessionDescription::TransportIndependentBandwidthType() { static PString s = "TIAS"; return s; }
 
 SDPSessionDescription::SDPSessionDescription(const OpalTransportAddress & address)
   : sessionName(SIP_DEFAULT_SESSION_NAME),
@@ -1015,7 +1088,6 @@ SDPSessionDescription::SDPSessionDescription(const OpalTransportAddress & addres
 void SDPSessionDescription::PrintOn(ostream & str) const
 {
   OpalTransportAddress connectionAddress(defaultConnectAddress);
-  PBoolean useCommonConnect = PTrue;
 
   // see common connect address is needed
   {
@@ -1035,11 +1107,12 @@ void SDPSessionDescription::PrintOn(ostream & str) const
       if ((descrMatched > matched))
         connectionAddress = descrAddress;
       else
-        useCommonConnect = PFalse;
+        connectionAddress.MakeEmpty();
     }
   }
 
-  // encode mandatory session information
+  /* encode mandatory session information, note the order is important according to RFC!
+     Must be vosiuepcbzkatrm and within the m it is icbka */
   str << "v=" << protocolVersion << "\r\n"
          "o=" << ownerUsername << ' '
       << ownerSessionId << ' '
@@ -1048,14 +1121,11 @@ void SDPSessionDescription::PrintOn(ostream & str) const
       << "\r\n"
          "s=" << sessionName << "\r\n";
 
-  // make sure the "c=" line (if required) is before the "t=" otherwise the proxy
-  // used by FWD will reject the call with an SDP parse error. This does not seem to be 
-  // required by the RFC so it is probably a bug in the proxy
-  if (useCommonConnect)
+  if (!connectionAddress.IsEmpty())
     str << "c=" << GetConnectAddressString(connectionAddress) << "\r\n";
   
-  str << "t=" << "0 0" << "\r\n"
-      << bandwidth;
+  str << bandwidth
+      << "t=" << "0 0" << "\r\n";
 
   switch (direction) {
     case SDPMediaDescription::RecvOnly:
@@ -1075,12 +1145,9 @@ void SDPSessionDescription::PrintOn(ostream & str) const
   }
 
   // encode media session information
-  PINDEX i;
-  for (i = 0; i < mediaDescriptions.GetSize(); i++) {
-    if (useCommonConnect) 
-      mediaDescriptions[i].PrintOn(connectionAddress, str);
-    else
-      str << mediaDescriptions[i];
+  for (PINDEX i = 0; i < mediaDescriptions.GetSize(); i++) {
+    if (mediaDescriptions[i].PreEncode())
+      mediaDescriptions[i].Encode(connectionAddress, str);
   }
 }
 
